/*********************************************************************\
       .

     .      
-                     .
,              
(    ).

   ,     ,  
             (.  tb_column).
                
    .

        0.    
.               .  
      .

		0.			1.
  0.	v[0,0]		v[0,1]	...
  1.	v[1,0]		v[1,1]	...
  ...

  

          (  1000 
- 100000).        . 
                .
,            
 .   ,      
       .      
  .

      ,      
        ,          
,                  (  
).
\*********************************************************************/

#ifdef __cplusplus

extern "C" {

#endif

#define TB_MAX_NAME_LENGTH	32

typedef unsigned char *tb_line;

struct tb_table
{
  //      
  int lines_count_mem;
  //    
  int lines_count;
  //      
  tb_line *lines;
  //    
  tb_line *lines_mem;

  //  
  int columns_count;
  struct tb_column * columns;
};

enum tb_type
{
	tb_type_int,			//    
	tb_type_double,			//     
	tb_type_char,			//   (  \0)
};

struct tb_column
{

/*       ,   .   -   
     TB_MAX_NAME_LENGTH -  1 . ,   
        .    
      tb_type.    
 ,            . 
                 .   
    ,        
       \0.        
tb_type_char.      . */

	char		name[TB_MAX_NAME_LENGTH];	//  
	enum tb_type		type;						//  
	int			size;						//  

  //   

  //        
  //     
  int offset;
};

void tb_initialize(struct tb_table *_t);
/*
	 

	_t	-    tb_table.

	    tb_table.   
	      tb_table   
	,         ,         
	      tb_...,    tb_initialize 
	         .   
	  tb_initialize     
	  ,              
	.        tb_initialize     tb_table     
	    :

							

		tb_table t;						struct tb_table t;

										tb_initialize(&t);
		...								...
		  t					  t
		...								...
*/

void tb_describe(struct tb_table *_t, struct tb_column *_col, int _n, int _occurs);
/*
	  

	_t			-  (. tb_initialize) ;
	_col[_n]	-   ;
	_n			-  ;
	_occurs		-  ,      (_occurs > 0).

	         _t.  
	      _col  (.  tb_column).    _occurs  
	         
	      .   tb_describe     
	        .
	_ocurs      ,      
	.             
	 .  _occurs        
	  ,      .

	      tb_describe       - 
	,      -  ,    .
	  ,              
	    ,   . */

int tb_insert(struct tb_table *_t, int _i);
/*
	  

	_t	- ,    (. tb_describe);
	_i	-   ,       (0 <= _i <= tb_getnrows(),
		  . tb_getnrows)

	          
	_i.       1.   
	-        .    _i   ==
	tb_getnrows(),          ( 
	 ). */

int tb_append(struct tb_table *_t);
/*
	     

	_t	- ,    (. tb_describe);

	   .

	        .  
	   tb_insert. */

int tb_getnrows(struct tb_table *_t);
/*
	     

	_t	- ,    (. tb_describe);

	   ,    .
*/

int tb_remove(struct tb_table *_t, int _i);
/*
	  

	_t	- ,    (. tb_describe);
	_i	-     (0 <= _i < tb_getnrows(), . tb_getnrows)

	         _i.       
	   1.    _i      ,  
	 -1,        _i.  
	(   ):

		while(tb_remove(_t, 0) >= 0)
			;
*/

int tb_setint(struct tb_table *_t, int _i, int _j, int _v);
/*
	    

	_t	- ,    (. tb_describe);
	_i	-   (0 <= _i < tb_getnrows());
	_j	-   (0 <= _j <  );
	_v	- .

	      _j   _i.  
	       tb_type_int. 
	_v. */

int tb_getint(struct tb_table *_t, int _i, int _j);
/*
	    

	_t	- ,    (. tb_describe);
	_i	-   (0 <= _i < tb_getnrows());
	_j	-   (0 <= _j <  ).

	       _j   _i.  
	       tb_type_int. */

double tb_setdouble(struct tb_table *_t, int _i, int _j, double _v);
/*
	    

	_t	- ,    (. tb_describe);
	_i	-   (0 <= _i < tb_getnrows());
	_j	-   (0 <= _j <  );
	_v	- .

	       _j   _i.  
	               tb_type_double.
	 _v. */

double tb_getdouble(struct tb_table *_t, int _i, int _j);
/*
	    

	_t	- ,    (. tb_describe);
	_i	-   (0 <= _i < tb_getnrows());
	_j	-   (0 <= _j <  ).

	        _j    _i. 
	       tb_type_double. */

char * tb_setchar(struct tb_table *_t, int _i, int _j, const char *_v);
/*
	    

	_t	- ,    (. tb_describe);
	_i	-   (0 <= _i < tb_getnrows());
	_j	-   (0 <= _j <  );
	_v	-     \0  .

	         _j    _i.  
	           tb_type_char. 
	strlen(_v) >=   (. tb_column),     
	 .  _v. */

char * tb_getchar(struct tb_table *_t, int _i, int _j, char *v, int _size);
/*
	    

	_t	- ,    (. tb_describe);
	_i	-   (0 <= _i < tb_getnrows());
	_j	-   (0 <= _j <  );
	_v	-  ;
	_size	-   _v (_size > 0).

	    _v        _j  
	_i.          tb_type_char.  
	_v        _size  ,       
	   \0.  _v. */

int tb_findcol(struct tb_table *_t, const char *_name);
/*
	   

	_t	- ,    (. tb_describe);
	_name	-  .

	      (  ,   
	,    -  ).
*/

void tb_clear(struct tb_table *_t);
/*
	   

	_t	- ,    (. tb_describe);

	  

		while(tb_remove(_t, 0) >= 0)
			;

	 ,     .
*/

void tb_discard(struct tb_table *_t);
/*
	 

	_t	- ,    (. tb_describe);

	  ,  .  ,     ,
	        tb_describe.
*/

// !! !void
int tb_match(struct tb_table *_t, struct tb_column *_col, int _n, int *_pos, int _exact);
/*
	  

	_t	- ,    (. tb_describe);
	_col[_n]	-   ;
	_n			-  ;
	_pos[_n]	-    (NULL - 
	_exact		-   .

	,                 
	  _col;   _exact 
  !=  1,      
	 .  1,    ,  -  0
	  . ,     (  ),
	   ,      .  
	 _pos != NULL, ,    1, pos[i] == j,   j
	-   _col[i]    */

void tb_getcolumn(struct tb_table *_t, int _j, struct tb_column *_col);
/*
	  

	_t	- ,    (. tb_describe);
	_j	-   (0 <= _j <  )
	_col	-    tb_column

	  _j-    _col.
*/

void tb_sort(struct tb_table *_t, int *_j, int _n, int *_o);
/*
	

	_t		- ,    (. tb_describe);
	_j[_n]	-    (0 <= _j[k] <  ,  0 <= k < _n);
	_n		-   ;
	_o[_n]	-  .

	     .  _o[k] >= 0,  
	      _j[k].   _o[k]
	< 0,        _j[k].

	 (    ):

		struct tb_table t;
		struct tb_column col[2];
		int j[2], o[2];

		tb_initialize(&t);

		strcpy(col[0].name, "INT");
		col[0].type = tb_type_int;
		strcpy(col[1].name, "DBL");
		col[1].type = tb_type_double;
		strcpy(col[2].name, "CHR");
		col[2].type = tb_type_char;
		col[2].size = 128;

		tb_describe(&t, col, 2, 1000);

		...
		 
		...
		j[0] = 0;	o[0] = 0;
		j[1] = 1;	o[1] = -1;
		tb_sort(&t, j, 2, o);
		...
*/

int tb_bsearch(struct tb_table *_t, int *_j, int _n, void *_d[]);
/*
	 

	_t		- ,    (. tb_describe);
	_j[_n]	-    (0 <= _j[k] <  ,  0 <= k < _n);
	_n		-   ;
	_d[_n]	-  .

	      ,    
	   _d. _d  -       ,
	      .              
	    (     _j, 
	tb_sort).         -1,     
	. ,       
	       .     
	  . 

	 (. tb_sort):

		void *d[2];
		int vi;
		double vd;
		int i, j[2];

		vi = 2;
		vd = 3.14;
		d[0] = &vi;
		d[1] = &vd;
		j[0] = 0;
		j[1] = 1;

		i = tb_bsearch(&t, j, 2, d);
		if(i >= 0)
		{
			...
			 i- 
			...
		}
		else
		{
			...
			 
			...
		}
*/

#ifdef __cplusplus

}

#endif
